
#ifndef COMPONENTREF_H
#define COMPONENTREF_H

#include <kore/componentproxy.h>
#include <iostream>

namespace kore
{
    class Interface;
    class MetaInfo;

    /**
     * Helper class used for passing korelib components around.
     * Use ComponentRef where you would normally use Interface*
     * for passing components as function parameters or return values.
     * This class also provides (via the ComponentProxy) lazy loading
     * instantiation of components/plugins as well as reference counting.
     */
    class KORE_API ComponentRef
    {
    public:
        /**
         * Default ctor. Used for creating an empty (invalid) ComponentRef.
         */
        ComponentRef()
        {
            _proxy = 0;
            //cerr << "ComponentRef(): " << this << endl;
        }
        /**
         * Creates a ComponentRef and sets the corresponding ComponentProxy
         * (component instance and metainfo).
         * Increments the reference count of the component (to be implemented).
         */
        ComponentRef( ComponentProxy* proxy )
        {
            _proxy = proxy;
            //cerr << "ComponentRef( proxy: " << proxy << " ): " << this << endl;
        }
        /**
         * Copy ctor.
         * Increments the reference count of the component (to be implemented).
         */
        ComponentRef( const ComponentRef& other )
        {
            _proxy = other._proxy;
            //cerr << "ComponentRef( other: " << &other << " ): " << this << endl;
        }
        /**
         * Dtor.
         * Decrements the reference count of the component and frees it when the
         * reference count gets to 0 (to be implemented).
         */
        ~ComponentRef()
        {
            //cerr << "~ComponentRef(): " << this << endl;
        }

        /**
         * Assignament operator.
         * Increments the reference count of the component (to be implemented).
         */
        ComponentRef& operator=( const ComponentRef& other )
        {
            if( this != &other )
                _proxy = other._proxy;
            return *this;
        }

        /**
         * Returns the component instance.
         * It also tries to load and instantiate the component if the instance is NULL.
         * @return the component instance or NULL if a instance could not be created.
         */
        Interface* instance() const
        {
            if( _proxy )
                return _proxy->instance();
            else
                return 0;
        }
        /**
         * Creates a new component instance.
         * It also tries to load the component plugin (library) if it wasn't loaded yet.
         * @return the newly created component instance or NULL if a instance could not be created.
         */
        Interface* newInstance() const
        {
            if( _proxy )
                return _proxy->newInstance();
            else
                return 0;
        }

        /**
         * Returns the component meta information, if any.
         * @return the component meta information or NULL if there is no metainfo
         */
        MetaInfo* metaInfo() const
        {
            if( _proxy )
                return _proxy->metaInfo();
            else
                return 0;
        }

        /**
         * Returns the component proxy, if any. This is an internal function.
         * @return the component proxy or NULL if there is no proxy.
         */
        ComponentProxy* proxy() const
        {
            return _proxy;
        }
        /**
         * Checks if the ComponentRef is valid.
         * A valid ComponentRef points to an actual component (via the ComponentProxy)
         * whereas an invalid ComponentRef has a special meaning of
         * "there is no actual component" (think NULL for pointers).
         * Invalid ComponentRef's are used in places (ie. function parameters or return values)
         * where a component is expected but there is no actual component.
         */
        bool isValid() const
        {
            return _proxy != 0;
        }
    private:
        ComponentProxy* _proxy;
    };
};

#endif
